@@ -43,6 +43,9 @@
 #include "builtin.h"
 #include "exec.h"
 #include "expand.h"
+#if YASH_ENABLE_HISTORY
+# include "history.h"
+#endif
 #include "input.h"
 #include "job.h"
 #include "option.h"
@@ -56,6 +59,10 @@
 #include "variable.h"
 #include "xfnmatch.h"
 #include "yash.h"
+#if YASH_ENABLE_LINEEDIT
+# include "lineedit/complete.h"
+# include "lineedit/lineedit.h"
+#endif
 
 
 /* way of command execution */
@@ -148,7 +155,8 @@ static void print_xtrace(void *const *argv);
 static void exec_fall_back_on_sh(
 	int argc, char *const *argv, char *const *env, const char *path)
     __attribute__((nonnull(2,4)));
-static void exec_function_body(command_T *body, void **args, bool finally_exit)
+static void exec_function_body(
+	command_T *body, void *const *args, bool finally_exit)
     __attribute__((nonnull));
 static inline int xexecv(const char *path, char *const *argv)
     __attribute__((nonnull(1)));
@@ -782,6 +790,7 @@ pid_t exec_process(
 	    finally_exit = true;
 	goto done3;
     }
+    print_xtrace(argv);
 
     /* find command path */
     if (cmdinfo.type == externalprogram) {
@@ -856,7 +865,7 @@ done:
  *   t_tstp: SIGTSTP is ignored if the parent is job-controlling
  *   t_leave: Don't clear traps and shellfds. Restore the signal mask for
  *          SIGCHLD. This option must be used iff the shell is going to `exec'
- *          to an extenal program.
+ *          to an external program.
  * Returns the return value of `fork'. */
 pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 {
@@ -899,12 +908,15 @@ pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 	if (sigtype & t_tstp)
 	    if (save_doing_job_control_now)
 		ignore_sigtstp();
-	restore_signals(sigtype & t_leave);  /* signal mask is restored here */
-	clear_shellfds(sigtype & t_leave);
 	if (!(sigtype & t_leave)) {
 	    clear_traps();
 	    neglect_all_jobs();
+#if YASH_ENABLE_HISTORY
+	    close_history_file();
+#endif
 	}
+	restore_signals(sigtype & t_leave);  /* signal mask is restored here */
+	clear_shellfds(sigtype & t_leave);
 	is_interactive_now = false;
     }
     return cpid;
@@ -1087,7 +1099,6 @@ void exec_simple_command(
 {
     assert(plcount(argv) == (size_t) argc);
 
-    print_xtrace(argv);
     switch (ci->type) {
     case externalprogram:
 	if (ci->ci_path == NULL) {
@@ -1134,7 +1145,7 @@ void exec_simple_command(
 	current_builtin_name = NULL;
 	break;
     case function:
-	exec_function_body(ci->ci_function, argv, finally_exit);
+	exec_function_body(ci->ci_function, argv + 1, finally_exit);
 	break;
     }
     if (finally_exit)
@@ -1156,7 +1167,11 @@ void print_xtrace(void *const *argv)
     bool tracevars = xtrace_buffer.contents != NULL
 		  && xtrace_buffer.length > 0;
 
-    if (shopt_xtrace && (tracevars || argv != NULL)) {
+    if (shopt_xtrace && (tracevars || argv != NULL)
+#if YASH_ENABLE_LINEEDIT
+	    && le_state != LE_STATE_ACTIVE
+#endif
+	    ) {
 	bool first = true;
 
 	struct promptset_T prompt = get_prompt(4);
@@ -1231,16 +1246,14 @@ void exec_fall_back_on_sh(
 
 /* Executes the specified command as a function.
  * `args' are the arguments to the function, which are wide strings cast to
- * (void *). `args[0]' is the function name and is ignored.
- * `args[1]' is the first argument, `args[2]' the second, and so on. */
-void exec_function_body(command_T *body, void **args, bool finally_exit)
+ * (void *). */
+void exec_function_body(command_T *body, void *const *args, bool finally_exit)
 {
     savefd_T *savefd;
 
-    assert(args[0] != NULL);
     if (open_redirections(body->c_redirs, &savefd)) {
 	open_new_environment(false);
-	set_positional_parameters(args + 1);
+	set_positional_parameters(args);
 	exec_nonsimple_command(body, finally_exit);
 	if (execinfo.exception == ee_return)
 	    execinfo.exception = ee_none;
@@ -1413,6 +1426,53 @@ int exec_variable_as_commands(const wchar_t *varname, const char *codename)
     return result;
 }
 
+#if YASH_ENABLE_LINEEDIT
+
+/* Generates completion candidates by executing the specified function.
+ * Returns the exit status of the function. */
+/* The prototype of this function is declared in "lineedit/complete.h". */
+int generate_candidates_using_function(
+	const wchar_t *funcname, le_context_T *context)
+{
+    if (funcname == NULL)
+	return Exit_SUCCESS;
+
+    le_compdebug("executing candidate generator function \"%ls\"", funcname);
+
+    command_T *function = get_function(funcname);
+    if (function == NULL) {
+	le_compdebug("  -- no such function");
+	return Exit_NOTFOUND;
+    }
+
+    struct parsestate_T *state = save_parse_state();
+
+    void *args[context->pwordc + 2];
+    memcpy(args, context->pwords, context->pwordc * sizeof *args);
+    args[context->pwordc] = context->origsrc;
+    args[context->pwordc + 1] = NULL;
+
+    int savelaststatus = laststatus, resultstatus;
+    bool saveposix = posixly_correct;
+    posixly_correct = false;
+
+    exec_function_body(function, args, false);
+
+    posixly_correct = saveposix;
+    resultstatus = laststatus;
+    laststatus = savelaststatus;
+
+    le_compdebug("finished executing function \"%ls\"", funcname);
+    if (resultstatus != Exit_SUCCESS)
+	le_compdebug("function returned exit status of %d", resultstatus);
+
+    restore_parse_state(state);
+
+    return resultstatus;
+}
+
+#endif /* YASH_ENABLE_LINEEDIT */
+
 
 /********** Builtins **********/
 
@@ -1713,11 +1773,10 @@ int dot_builtin(int argc, void **argv)
 	set_positional_parameters(argv + xoptind);
     }
 
-    FILE *f = fopen(path, "r");
+    int fd = move_to_shellfd(open(path, O_RDONLY));
     if (path != mbsfilename)
 	free(path);
-    f = reopen_with_shellfd(f, "r", true);
-    if (!f) {
+    if (fd < 0) {
 	xerror(errno, Ngt("cannot open `%s'"), mbsfilename);
 	free(mbsfilename);
 	if (!is_interactive)
@@ -1725,8 +1784,9 @@ int dot_builtin(int argc, void **argv)
 	return Exit_FAILURE;
     }
 
-    exec_input(f, mbsfilename, false, enable_alias, false);
-    fclose(f);
+    exec_input(fd, mbsfilename, false, enable_alias, false);
+    remove_shellfd(fd);
+    xclose(fd);
     free(mbsfilename);
 
     if (has_args) {
