@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* job.c: job control */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,12 @@
 #include "strbuf.h"
 #include "util.h"
 #include "yash.h"
-#if YASH_ENABLE_LINEEDIT && !defined(FG_DONT_SAVE_TERMINAL)
-# include "lineedit/terminfo.h"
+#if YASH_ENABLE_LINEEDIT
+# include "xfnmatch.h"
+# include "lineedit/complete.h"
+# if !defined(FG_DONT_SAVE_TERMINAL)
+#  include "lineedit/terminfo.h"
+# endif
 #endif
 
 
@@ -335,19 +339,31 @@ void do_wait(void)
 {
     pid_t pid;
     int status;
+#if HAVE_WCONTINUED
+    static int waitpidoption = WUNTRACED | WCONTINUED | WNOHANG;
+#else
+    const int waitpidoption = WUNTRACED | WNOHANG;
+#endif
 
 start:
-#ifdef HAVE_WCONTINUED
-    pid = waitpid(-1, &status, WUNTRACED | WCONTINUED | WNOHANG);
-#else
-    pid = waitpid(-1, &status, WUNTRACED | WNOHANG);
-#endif
+    pid = waitpid(-1, &status, waitpidoption);
     if (pid < 0) {
 	switch (errno) {
 	    case EINTR:
 		goto start;  /* try again */
 	    case ECHILD:
 		return;      /* there are no child processes */
+#if HAVE_WCONTINUED
+	    /* Even when the WCONTINUED flag is defined in the <sys/wait.h>
+	     * header, the OS kernel may not support it. We try again without
+	     * the flag if it is rejected. */
+	    case EINVAL:
+		if (waitpidoption & WCONTINUED) {
+		    waitpidoption &= ~WCONTINUED;
+		    goto start;
+		}
+		/* falls thru! */
+#endif
 	    default:
 		xerror(errno, "waitpid");
 		return;
@@ -569,14 +585,16 @@ int calc_status(int status)
 {
     if (WIFEXITED(status))
 	return WEXITSTATUS(status);
+#ifdef WIFCONTINUED
+    if (WIFCONTINUED(status))
+	return Exit_SUCCESS;
+    /* On FreeBSD, when WIFCONTINUED is true, WIFSIGNALED is also true. We must
+     * be careful about the order of these checks. */
+#endif
     if (WIFSIGNALED(status))
 	return WTERMSIG(status) + TERMSIGOFFSET;
     if (WIFSTOPPED(status))
 	return WSTOPSIG(status) + TERMSIGOFFSET;
-#ifdef WIFCONTINUED
-    if (WIFCONTINUED(status))
-	return Exit_SUCCESS;
-#endif
     assert(false);
 }
 
@@ -878,6 +896,55 @@ found:
     return jobnumber;
 }
 
+#if YASH_ENABLE_LINEEDIT
+
+/* Generates completion candidates for job names that match the glob pattern
+ * in the specified context. */
+/* The prototype of this function is declared in "lineedit/complete.h". */
+void generate_job_candidates(le_candgentype_T type, le_context_T *context)
+{
+    if (!(type & CGT_JOB))
+	return;
+
+    const wchar_t *pattern = context->pattern;
+    if (pattern[0] == L'%')
+	pattern += 1;
+    else if (pattern[0] == L'\\' && pattern[1] == L'%')
+	pattern += 2;
+    le_compdebug("adding jobs for pattern \"%ls\"", pattern);
+
+    xfnmatch_T *xfnm = xfnm_compile(pattern, XFNM_HEADONLY | XFNM_TAILONLY);
+    if (xfnm == NULL)
+	return;
+
+    for (size_t i = 1; i < joblist.length; i++) {
+	const job_T *job = joblist.contents[i];
+	if (job == NULL)
+	    continue;
+	switch (job->j_status) {
+	    case JS_RUNNING:  if (!(type & CGT_RUNNING)) continue;  break;
+	    case JS_STOPPED:  if (!(type & CGT_STOPPED)) continue;  break;
+	    case JS_DONE:     if (!(type & CGT_DONE))    continue;  break;
+	}
+
+	wchar_t *jobname = get_job_name(job);
+	if (xfnm_wmatch(xfnm, jobname).start != (size_t) -1) {
+	    wchar_t *cand;
+	    if (context->src[0] != L'\0' && context->src[0] != L'%')
+		cand = xwcsdup(jobname);
+	    else
+		cand = malloc_wprintf(L"%%%ls", jobname);
+	    le_new_candidate(CT_JOB, cand, malloc_wprintf(L"%%%zu", i));
+	}
+
+	if (jobname != job->j_procs[0].pr_name)
+	    free(jobname);
+    }
+    xfnm_free(xfnm);
+}
+
+#endif /* YASH_ENABLE_LINEEDIT */
+
 
 /********** Builtins **********/
 
