@@ -49,7 +49,6 @@ static void pipesfree(pipeline_T *p);
 static void ifcmdsfree(ifcommand_T *i);
 static void caseitemsfree(caseitem_T *i);
 static void wordfree_vp(void *w);
-static void paramfree(paramexp_T *p);
 static void assignsfree(assign_T *a);
 static void redirsfree(redir_T *r);
 static void embedcmdfree(embedcmd_T c);
@@ -235,8 +234,6 @@ void embedcmdfree(embedcmd_T c)
 
 /********** Auxiliary Functions for Parser **********/
 
-static bool is_portable_name_char(wchar_t c)
-    __attribute__((const));
 static wchar_t *skip_name(const wchar_t *s)
     __attribute__((pure,nonnull));
 
@@ -292,9 +289,10 @@ bool is_name(const wchar_t *s)
 bool is_keyword(const wchar_t *s)
 {
     /* List of keywords:
-     *    case do done elif else esac fi for if in then until while { } !
+     *    case do done elif else esac fi for function if in then until while
+     *    { } !
      * The following words are currently not keywords:
-     *    [[ ]] function select */
+     *    select [[ ]] */
     switch (s[0]) {
 	case L'c':
 	    return s[1] == L'a' && s[2] == L's' && s[3] == L'e' && s[4]== L'\0';
@@ -309,7 +307,10 @@ bool is_keyword(const wchar_t *s)
 		&& s[4] == L'\0';
 	case L'f':
 	    return (s[1] == L'i' && s[2] == L'\0')
-		|| (s[1] == L'o' && s[2] == L'r' && s[3] == L'\0');
+		|| (s[1] == L'o' && s[2] == L'r' && s[3] == L'\0')
+		|| (s[1] == L'u' && s[2] == L'n' && s[3] == L'c'
+			&& s[4] == L't' && s[5] == L'i' && s[6] == L'o'
+			&& s[7] == L'n' && s[8] == L'\0');
 	case L'i':
 	    return (s[1] == L'f' || s[1] == L'n') && s[2] == L'\0';
 	case L't':
@@ -542,8 +543,10 @@ parseresult_T read_and_parse(
     cindex = 0;
     wb_init(&cbuf);
 
-    if (info->intrinput)
-	((struct input_readline_info *) info->inputinfo)->type = 1;
+    if (info->intrinput) {
+	struct input_interactive_info *intrinfo = info->inputinfo;
+	intrinfo->prompttype = 1;
+    }
 
     cinfo->lastinputresult = INPUT_OK;
     switch (read_more_input()) {
@@ -1368,6 +1371,7 @@ reparse:
 	wchar_t *endofheredoc = parse_word_as_wcs();
 	if (endofheredoc[0] == L'\0') {
 	    serror(Ngt("here-document delimiter missing"));
+	    free(endofheredoc);
 	    free(result);
 	    return NULL;
 	}
