@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* sig.c: signal handling */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,8 @@
 #include "util.h"
 #include "yash.h"
 #if YASH_ENABLE_LINEEDIT
+# include "xfnmatch.h"
+# include "lineedit/complete.h"
 # include "lineedit/lineedit.h"
 #endif
 
@@ -537,7 +539,8 @@ int wait_for_sigchld(bool interruptible, bool return_on_trap)
 /* Waits for the specified file descriptor to be available for reading.
  * `handle_sigchld' and `handle_sigwinch' are called to handle SIGCHLD and
  * SIGWINCH that are caught while waiting.
- * If `trap' is true, traps are also handled while waiting.
+ * If `trap' is true, traps are also handled while waiting and the
+ * `sigint_received' flag is cleared when this function returns.
  * The maximum time length of wait is specified by `timeout' in milliseconds.
  * If `timeout' is negative, the wait time is unlimited.
  * If the wait is interrupted by a signal, this function will re-wait for the
@@ -553,10 +556,12 @@ bool wait_for_input(int fd, bool trap, int timeout)
 
     ss = accept_sigmask;
     sigdelset(&ss, SIGCHLD);
+    if (interactive_handlers_set) {
+	sigdelset(&ss, SIGINT);
 #if YASH_ENABLE_LINEEDIT && defined(SIGWINCH)
-    if (interactive_handlers_set)
 	sigdelset(&ss, SIGWINCH);
 #endif
+    }
 
     if (timeout < 0) {
 	top = NULL;
@@ -578,6 +583,8 @@ bool wait_for_input(int fd, bool trap, int timeout)
 	FD_ZERO(&fdset);
 	FD_SET(fd, &fdset);
 	if (pselect(fd + 1, &fdset, NULL, NULL, top, &ss) >= 0) {
+	    if (trap)
+		sigint_received = false;
 	    return FD_ISSET(fd, &fdset);
 	} else {
 	    if (errno != EINTR) {
@@ -627,6 +634,7 @@ int handle_traps(void)
 	return 0;
 
     int signum = 0;
+    bool save_sigint_received = sigint_received;
     struct parsestate_T *state = NULL;
     savelaststatus = laststatus;
 
@@ -643,13 +651,15 @@ int handle_traps(void)
 #if YASH_ENABLE_LINEEDIT
 		    le_suspend_readline();
 #endif
+		    save_sigint_received |= sigint_received;
+		    sigint_received = false;
 		    if (!state)
 			state = save_parse_state();
 		    signum = handled_signal = s->no;
+		    command = xwcsdup(command);
 		    exec_wcs(command, "trap", false);
+		    free(command);
 		    laststatus = savelaststatus;
-		    if (command != trap_command[i])
-			free(command);
 		}
 	    }
 	}
@@ -665,19 +675,22 @@ int handle_traps(void)
 #if YASH_ENABLE_LINEEDIT
 		    le_suspend_readline();
 #endif
+		    save_sigint_received |= sigint_received;
+		    sigint_received = false;
 		    if (!state)
 			state = save_parse_state();
 		    signum = handled_signal = sigrtmin + i;
+		    command = xwcsdup(command);
 		    exec_wcs(command, "trap", false);
+		    free(command);
 		    laststatus = savelaststatus;
-		    if (command != rttrap_command[i])
-			free(command);
 		}
 	    }
 	}
 #endif  /* defined SIGRTMAX && defined SIGRTMIN */
     } while (any_signal_received);
 
+    sigint_received |= save_sigint_received;
     savelaststatus = -1;
     handled_signal = -1;
     if (state)
@@ -699,12 +712,10 @@ void execute_exit_trap(void)
 	assert(!exit_handled);
 	exit_handled = true;
 	savelaststatus = laststatus;
+	command = xwcsdup(command);
 	exec_wcs(command, "EXIT trap", false);
+	free(command);
 	savelaststatus = -1;
-#ifndef NDEBUG
-	if (command != trap_command[sigindex(0)])
-	    free(command);
-#endif
     }
 }
 
@@ -752,9 +763,7 @@ void set_trap(int signum, const wchar_t *command)
 	return;
     }
 
-    /* If `*commandp' is currently executed, we must not free it. */
-    if (signum != handled_signal && (signum != 0 || !exit_handled))
-	free(*commandp);
+    free(*commandp);
     if (command) {
 	if (command[0] != L'\0')
 	    any_trap_set = true;
@@ -885,20 +894,17 @@ void set_interrupted(void)
 }
 
 /* Resets the `sigint_received' flag. */
+/* This function should be called just after calling `handle_signals'. */
 void reset_sigint(void)
 {
-    if (interactive_handlers_set) {
-	sigset_t ss, savess;
-	sigemptyset(&ss);
-	sigemptyset(&savess);
-	sigaddset(&ss, SIGINT);
-	sigprocmask(SIG_UNBLOCK, &ss, &savess);
-	sigprocmask(SIG_SETMASK, &savess, NULL);
-    }
     sigint_received = false;
 }
 
 #if YASH_ENABLE_LINEEDIT
+
+static void sig_new_candidate(const xfnmatch_T *pat, int num, xwcsbuf_T *name)
+    __attribute__((nonnull));
+
 #ifdef SIGWINCH
 
 /* If SIGWINCH has been caught and line-editing is currently active, cause
@@ -919,6 +925,70 @@ void reset_sigwinch(void)
 #endif
 }
 
+/* Generates completion candidates for signal names that match the glob pattern
+ * in the specified context. */
+/* The prototype of this function is declared in "lineedit/complete.h". */
+void generate_signal_candidates(le_candgentype_T type, le_context_T *context)
+{
+    if (!(type & CGT_SIGNAL))
+	return;
+
+    le_compdebug("adding signals for pattern \"%ls\"", context->pattern);
+    if (!le_compile_cpattern(context))
+	return;
+
+    bool prefix = matchwcsprefix(context->src, L"SIG");
+    xwcsbuf_T buf;
+    wb_init(&buf);
+    for (const signal_T *s = signals; s->no; s++) {
+	if (prefix)
+	    wb_cat(&buf, L"SIG");
+	wb_cat(&buf, s->name);
+	sig_new_candidate(context->cpattern, s->no, &buf);
+    }
+#if defined SIGRTMIN && defined SIGRTMAX
+    int sigrtmin = SIGRTMIN, sigrtmax = SIGRTMAX;
+    for (int s = sigrtmin; s <= sigrtmax; s++) {
+	if (prefix)
+	    wb_cat(&buf, L"SIG");
+	wb_cat(&buf, L"RTMIN");
+	if (s != sigrtmin)
+	    wb_wprintf(&buf, L"+%d", s - sigrtmin);
+	sig_new_candidate(context->cpattern, s, &buf);
+
+	if (prefix)
+	    wb_cat(&buf, L"SIG");
+	wb_cat(&buf, L"RTMAX");
+	if (s != sigrtmax)
+	    wb_wprintf(&buf, L"-%d", sigrtmax - s);
+	sig_new_candidate(context->cpattern, s, &buf);
+    }
+#endif
+    wb_destroy(&buf);
+}
+
+/* If the specified pattern matches the specified name, adds a new completion
+ * candidate with the specified name and the description for the specified
+ * signal. */
+void sig_new_candidate(const xfnmatch_T *pat, int num, xwcsbuf_T *name)
+{
+    if (xfnm_wmatch(pat, name->contents).start != (size_t) -1) {
+	xwcsbuf_T desc;
+	wb_init(&desc);
+#if HAVE_STRSIGNAL
+	char *mbsdesc = strsignal(num);
+	if (mbsdesc != NULL)
+	    wb_wprintf(&desc, L"%d: %s", num, mbsdesc);
+	else
+#endif
+	    wb_wprintf(&desc, L"%d", num);
+	le_new_candidate(CT_SIG, wb_towcs(name), wb_towcs(&desc));
+	wb_init(name);
+    } else {
+	wb_clear(name);
+    }
+}
+
 #endif /* YASH_ENABLE_LINEEDIT */
 
 
