@@ -41,13 +41,15 @@ extern xstrbuf_T *sb_initwith(xstrbuf_T *restrict buf, char *restrict s)
     __attribute__((nonnull));
 static inline void sb_destroy(xstrbuf_T *buf)
     __attribute__((nonnull));
-extern char *sb_tostr(xstrbuf_T *buf)
+static inline char *sb_tostr(xstrbuf_T *buf)
     __attribute__((nonnull));
 extern xstrbuf_T *sb_setmax(xstrbuf_T *buf, size_t newmax)
     __attribute__((nonnull));
 extern xstrbuf_T *sb_ensuremax(xstrbuf_T *buf, size_t max)
     __attribute__((nonnull));
-extern xstrbuf_T *sb_clear(xstrbuf_T *buf)
+static inline xstrbuf_T *sb_truncate(xstrbuf_T *buf, size_t newlength)
+    __attribute__((nonnull));
+static inline xstrbuf_T *sb_clear(xstrbuf_T *buf)
     __attribute__((nonnull));
 extern xstrbuf_T *sb_replace_force(
 	xstrbuf_T *restrict buf, size_t i, size_t bn,
@@ -87,20 +89,15 @@ extern xstrbuf_T *sb_ccat_repeat(xstrbuf_T *buf, char c, size_t n)
 extern _Bool sb_wccat(
 	xstrbuf_T *restrict buf, wchar_t c, mbstate_t *restrict ps)
     __attribute__((nonnull));
-extern wchar_t *sb_wcscat(
-	xstrbuf_T *restrict buf,
+extern wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
 	const wchar_t *restrict s, mbstate_t *restrict ps)
-    __attribute__((nonnull(1)));
+    __attribute__((nonnull));
 extern int sb_vprintf(
 	xstrbuf_T *restrict buf, const char *restrict format, va_list ap)
     __attribute__((nonnull(1,2),format(printf,2,0)));
 extern int sb_printf(
 	xstrbuf_T *restrict buf, const char *restrict format, ...)
     __attribute__((nonnull(1,2),format(printf,2,3)));
-extern size_t sb_strftime(
-	xstrbuf_T *restrict buf,
-	const char *restrict format, const struct tm *restrict tm)
-    __attribute__((nonnull,format(strftime,2,0)));
 
 extern xwcsbuf_T *wb_init(xwcsbuf_T *buf)
     __attribute__((nonnull));
@@ -108,13 +105,15 @@ extern xwcsbuf_T *wb_initwith(xwcsbuf_T *restrict buf, wchar_t *restrict s)
     __attribute__((nonnull));
 static inline void wb_destroy(xwcsbuf_T *buf)
     __attribute__((nonnull));
-extern wchar_t *wb_towcs(xwcsbuf_T *buf)
+static inline wchar_t *wb_towcs(xwcsbuf_T *buf)
     __attribute__((nonnull));
 extern xwcsbuf_T *wb_setmax(xwcsbuf_T *buf, size_t newmax)
     __attribute__((nonnull));
 extern xwcsbuf_T *wb_ensuremax(xwcsbuf_T *buf, size_t max)
     __attribute__((nonnull));
-extern xwcsbuf_T *wb_clear(xwcsbuf_T *buf)
+static inline xwcsbuf_T *wb_truncate(xwcsbuf_T *buf, size_t newlength)
+    __attribute__((nonnull));
+static inline xwcsbuf_T *wb_clear(xwcsbuf_T *buf)
     __attribute__((nonnull));
 extern xwcsbuf_T *wb_replace_force(
 	xwcsbuf_T *restrict buf, size_t i, size_t bn,
@@ -162,36 +161,63 @@ extern int wb_wprintf(
 
 extern char *malloc_wcsntombs(const wchar_t *s, size_t n)
     __attribute__((nonnull,malloc,warn_unused_result));
-static inline char *malloc_wcstombs(const wchar_t *s)
+extern char *malloc_wcstombs(const wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static inline char *realloc_wcstombs(wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 extern wchar_t *malloc_mbsntowcs(const char *s, size_t n)
     __attribute__((nonnull,malloc,warn_unused_result));
-static inline wchar_t *malloc_mbstowcs(const char *s)
+extern wchar_t *malloc_mbstowcs(const char *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static inline wchar_t *realloc_mbstowcs(char *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 
 static inline char *malloc_vprintf(const char *format, va_list ap)
     __attribute__((nonnull(1),malloc,warn_unused_result,format(printf,1,0)));
-static inline char *malloc_printf(const char *format, ...)
+extern char *malloc_printf(const char *format, ...)
     __attribute__((nonnull(1),malloc,warn_unused_result,format(printf,1,2)));
 static inline wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
     __attribute__((nonnull(1),malloc,warn_unused_result));
-static inline wchar_t *malloc_wprintf(const wchar_t *format, ...)
+extern wchar_t *malloc_wprintf(const wchar_t *format, ...)
     __attribute__((nonnull(1),malloc,warn_unused_result));
 
 
-/* Frees a multibyte string buffer, abandoning the contents. */
+/* Frees the specified multibyte string buffer. The contents are lost. */
 void sb_destroy(xstrbuf_T *buf)
 {
     free(buf->contents);
 }
 
-/* Inserts the first `n' bytes of a multibyte string `s' at the offset `i'
- * in the buffer `buf'.
- * No boundary checks are done and a null character is not considered special.
+/* Frees the specified multibyte string buffer and returns the contents.
+ * The caller must `free' the return value. */
+char *sb_tostr(xstrbuf_T *buf)
+{
+    return buf->contents;
+}
+
+/* Shrinks the length of the buffer to `newlength'.
+ * `newlength' must not be larger than the current length.
+ * Characters beyond the new length are lost.
+ * `maxlength' of the buffer is not changed. */
+xstrbuf_T *sb_truncate(xstrbuf_T *buf, size_t newlength)
+{
+#ifdef assert
+    assert(newlength <= buf->length);
+#endif
+    buf->contents[buf->length = newlength] = '\0';
+    return buf;
+}
+
+/* Clears the contents of the specified string buffer.
+ * `maxlength' of the buffer is not changed. */
+xstrbuf_T *sb_clear(xstrbuf_T *buf)
+{
+    return sb_truncate(buf, 0);
+}
+
+/* Inserts the first `n' bytes of multibyte string `s' at offset `i' in buffer
+ * `buf'.
+ * No boundary checks are done and null characters are not considered special.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_ninsert_force(
 	xstrbuf_T *restrict buf, size_t i, const char *restrict s, size_t n)
@@ -199,8 +225,8 @@ xstrbuf_T *sb_ninsert_force(
     return sb_replace_force(buf, i, 0, s, n);
 }
 
-/* Inserts the first `n' bytes of a multibyte string `s' at the offset `i'
- * in the buffer `buf'.
+/* Inserts the first `n' bytes of multibyte string `s' at offset `i' in buffer
+ * `buf'.
  * If (strlen(s) <= n), the whole of `s' is inserted.
  * If (buf->length <= i), `s' is appended to the end of the buffer.
  * `s' must not be part of `buf->contents'. */
@@ -210,7 +236,7 @@ xstrbuf_T *sb_ninsert(
     return sb_replace(buf, i, 0, s, n);
 }
 
-/* Inserts a multibyte string `s' at the offset `i' in the buffer `buf'.
+/* Inserts multibyte string `s' at offset `i' in buffer `buf'.
  * If (buf->length <= i), `s' is appended to the end of the buffer.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_insert(xstrbuf_T *restrict buf, size_t i, const char *restrict s)
@@ -218,8 +244,8 @@ xstrbuf_T *sb_insert(xstrbuf_T *restrict buf, size_t i, const char *restrict s)
     return sb_replace(buf, i, 0, s, Size_max);
 }
 
-/* Appends the first `n' bytes of a multibyte string `s' to the buffer `buf'.
- * No boundary checks are done and a null character is not considered special.
+/* Appends the first `n' bytes of multibyte string `s' to buffer `buf'.
+ * No boundary checks are done and null characters are not considered special.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_ncat_force(
 	xstrbuf_T *restrict buf, const char *restrict s, size_t n)
@@ -227,7 +253,7 @@ xstrbuf_T *sb_ncat_force(
     return sb_replace_force(buf, buf->length, 0, s, n);
 }
 
-/* Appends the first `n' bytes of a multibyte string `s' to the buffer `buf'.
+/* Appends the first `n' bytes of multibyte string `s' to buffer `buf'.
  * If (strlen(s) <= n), the whole of `s' is appended.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_ncat(xstrbuf_T *restrict buf, const char *restrict s, size_t n)
@@ -235,14 +261,14 @@ xstrbuf_T *sb_ncat(xstrbuf_T *restrict buf, const char *restrict s, size_t n)
     return sb_replace(buf, Size_max, 0, s, n);
 }
 
-/* Appends a multibyte string `s' to the buffer `buf'.
+/* Appends multibyte string `s' to buffer `buf'.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_cat(xstrbuf_T *restrict buf, const char *restrict s)
 {
     return sb_replace(buf, Size_max, 0, s, Size_max);
 }
 
-/* Appends a multibyte string `s' to the buffer `buf' and free the string.
+/* Appends multibyte string `s' to buffer `buf' and free the string.
  * `s' must not be part of `buf->contents'. */
 xstrbuf_T *sb_catfree(xstrbuf_T *restrict buf, char *restrict s)
 {
@@ -251,7 +277,7 @@ xstrbuf_T *sb_catfree(xstrbuf_T *restrict buf, char *restrict s)
     return buf;
 }
 
-/* Removes `n' bytes at the offset `i' in the buffer `buf'.
+/* Removes `n' bytes at offset `i' in buffer `buf'.
  * If (buf->length <= i), `buf' is unchanged.
  * If (buf->length <= i + n), `buf' is truncated to `i' bytes. */
 xstrbuf_T *sb_remove(xstrbuf_T *buf, size_t i, size_t n)
@@ -259,15 +285,42 @@ xstrbuf_T *sb_remove(xstrbuf_T *buf, size_t i, size_t n)
     return sb_replace(buf, i, n, "", 0);
 }
 
-/* Frees a wide string buffer, abandoning the contents. */
+/* Frees the specified wide string buffer. The contents are lost. */
 void wb_destroy(xwcsbuf_T *buf)
 {
     free(buf->contents);
 }
 
-/* Inserts the first `n' characters of a wide string `s'
- * at the offset `i' in the buffer `buf'.
- * No boundary checks are done and a null character is not considered special.
+/* Frees the specified wide string buffer and returns the contents.
+ * The caller must `free' the return value. */
+wchar_t *wb_towcs(xwcsbuf_T *buf)
+{
+    return buf->contents;
+}
+
+/* Shrinks the length of the specified buffer to `newlength'.
+ * `newlength' must not be larger than the current length.
+ * Characters beyond the new length are lost.
+ * `maxlength' of the buffer is not changed. */
+xwcsbuf_T *wb_truncate(xwcsbuf_T *buf, size_t newlength)
+{
+#ifdef assert
+    assert(newlength <= buf->length);
+#endif
+    buf->contents[buf->length = newlength] = L'\0';
+    return buf;
+}
+
+/* Clears the contents of the specified string buffer.
+ * `maxlength' of the buffer is not changed. */
+xwcsbuf_T *wb_clear(xwcsbuf_T *buf)
+{
+    return wb_truncate(buf, 0);
+}
+
+/* Inserts the first `n' characters of wide string `s' at offset `i' in buffer
+ * `buf'.
+ * No boundary checks are done and null characters are not considered special.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_ninsert_force(
 	xwcsbuf_T *restrict buf, size_t i, const wchar_t *restrict s, size_t n)
@@ -275,8 +328,8 @@ xwcsbuf_T *wb_ninsert_force(
     return wb_replace_force(buf, i, 0, s, n);
 }
 
-/* Inserts the first `n' characters of a wide string
- * at the offset `i' in the buffer `buf'.
+/* Inserts the first `n' characters of wide string `s` at offset `i' in buffer
+ * `buf'.
  * If (wcslen(s) <= n), the whole of `s' is inserted.
  * If (buf->length <= i), `s' is appended to the end of the buffer.
  * `s' must not be part of `buf->contents'. */
@@ -286,7 +339,7 @@ xwcsbuf_T *wb_ninsert(
     return wb_replace(buf, i, 0, s, n);
 }
 
-/* Inserts a wide string at the offset `i' in the buffer `buf'.
+/* Inserts wide string `s' at offset `i' in buffer `buf'.
  * If (buf->length <= i), `s' is appended to the end of the buffer.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_insert(
@@ -295,8 +348,8 @@ xwcsbuf_T *wb_insert(
     return wb_replace(buf, i, 0, s, Size_max);
 }
 
-/* Appends the first `n' characters of a wide string `s' to the buffer `buf'.
- * No boundary checks are done and a null character is not considered special.
+/* Appends the first `n' characters of wide string `s' to buffer `buf'.
+ * No boundary checks are done and null characters are not considered special.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_ncat_force(
 	xwcsbuf_T *restrict buf, const wchar_t *restrict s, size_t n)
@@ -304,7 +357,7 @@ xwcsbuf_T *wb_ncat_force(
     return wb_replace_force(buf, buf->length, 0, s, n);
 }
 
-/* Appends the first `n' characters of a wide string `s' to the buffer `buf'.
+/* Appends the first `n' characters of wide string `s' to buffer `buf'.
  * If (wcslen(s) <= n), the whole of `s' is appended.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_ncat(xwcsbuf_T *restrict buf, const wchar_t *restrict s, size_t n)
@@ -312,14 +365,14 @@ xwcsbuf_T *wb_ncat(xwcsbuf_T *restrict buf, const wchar_t *restrict s, size_t n)
     return wb_replace(buf, Size_max, 0, s, n);
 }
 
-/* Appends a wide string `s' to the buffer `buf'.
+/* Appends wide string `s' to buffer `buf'.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_cat(xwcsbuf_T *restrict buf, const wchar_t *restrict s)
 {
     return wb_replace(buf, Size_max, 0, s, Size_max);
 }
 
-/* Appends a wide string `s' to the buffer and free the string.
+/* Appends wide string `s' to buffer and frees the string.
  * `s' must not be part of `buf->contents'. */
 xwcsbuf_T *wb_catfree(xwcsbuf_T *restrict buf, wchar_t *restrict s)
 {
@@ -328,7 +381,7 @@ xwcsbuf_T *wb_catfree(xwcsbuf_T *restrict buf, wchar_t *restrict s)
     return buf;
 }
 
-/* Removes `n' characters at the offset `i' in the buffer `buf'.
+/* Removes `n' characters at offset `i' in buffer `buf'.
  * If (buf->length <= i), `buf' is unchanged.
  * If (buf->length <= i + n), `buf' is truncated to `i' characters. */
 xwcsbuf_T *wb_remove(xwcsbuf_T *buf, size_t i, size_t n)
@@ -336,16 +389,8 @@ xwcsbuf_T *wb_remove(xwcsbuf_T *buf, size_t i, size_t n)
     return wb_replace(buf, i, n, L"", 0);
 }
 
-/* Converts a wide string to a newly malloced multibyte string.
- * Returns NULL on error.
- * The resulting string starts and ends in the initial shift state.*/
-char *malloc_wcstombs(const wchar_t *s)
-{
-    return malloc_wcsntombs(s, Size_max);
-}
-
-/* Converts a wide string to a newly malloced multibyte string and free the
- * wide string.
+/* Converts the specified wide string into a newly malloced multibyte string and
+ * frees the original wide string.
  * Returns NULL on error. The wide string is freed anyway.
  * The resulting string starts and ends in the initial shift state.*/
 char *realloc_wcstombs(wchar_t *s)
@@ -355,15 +400,8 @@ char *realloc_wcstombs(wchar_t *s)
     return result;
 }
 
-/* Converts a multibyte string to a newly malloced wide string.
- * Returns NULL on error. */
-wchar_t *malloc_mbstowcs(const char *s)
-{
-    return malloc_mbsntowcs(s, Size_max);
-}
-
-/* Converts a multibyte string to a newly malloced wide string and free the
- * multibyte string.
+/* Converts the specified multibyte string into a newly malloced wide string and
+ * frees the multibyte string.
  * Returns NULL on error. The multibyte string is freed anyway. */
 wchar_t *realloc_mbstowcs(char *s)
 {
@@ -381,17 +419,6 @@ char *malloc_vprintf(const char *format, va_list ap)
     return sb_tostr(&buf);
 }
 
-/* Returns the result of `sprintf' as a newly malloced string. */
-char *malloc_printf(const char *format, ...)
-{
-    va_list ap;
-    char *result;
-    va_start(ap, format);
-    result = malloc_vprintf(format, ap);
-    va_end(ap);
-    return result;
-}
-
 /* Returns the result of `vswprintf' as a newly malloced string. */
 wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
 {
@@ -401,17 +428,6 @@ wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
     return wb_towcs(&buf);
 }
 
-/* Returns the result of `swprintf' as a newly malloced string. */
-wchar_t *malloc_wprintf(const wchar_t *format, ...)
-{
-    va_list ap;
-    wchar_t *result;
-    va_start(ap, format);
-    result = malloc_vwprintf(format, ap);
-    va_end(ap);
-    return result;
-}
-
 
 #undef Size_max
 
